var laya = (function (exports) {
    'use strict';

    class GameEvent {
    }
    GameEvent.GAME_WIN = "game_win";
    GameEvent.GAME_FAIL = "game_fail";
    GameEvent.GAME_EXIT = "game_exit";

    class BlockWarCfg {
        constructor() {
            this.ai_second = [10, 11, 12, 13, 14, 15];
        }
    }
    class CfgMgr {
        static parseCfgData(cfgData) {
            CfgMgr.levelMap = {};
            let levels = cfgData['levels'];
            for (let i = 0; i < levels.length; i++) {
                let cfg = new BlockWarCfg();
                cfg.level = levels[i].level;
                cfg.map_id = levels[i].map_id;
                let ai_second = levels[i].ai_second.split(',');
                let start = parseInt(ai_second[0]);
                let end = parseInt(ai_second[1]);
                cfg.ai_second = [];
                for (let i = start; i <= end; i++) {
                    cfg.ai_second.push(i);
                }
                CfgMgr.levelMap[cfg.level] = cfg;
            }
        }
        static getLevelCfg(level) {
            return CfgMgr.levelMap[level];
        }
    }

    var Scene = Laya.Scene;
    var REG = Laya.ClassUtils.regClass;
    var ui;
    (function (ui) {
        var mini_game;
        (function (mini_game) {
            class GameUI extends Scene {
                constructor() { super(); }
                createChildren() {
                    super.createChildren();
                    this.loadScene("mini_game/Game");
                }
            }
            mini_game.GameUI = GameUI;
            REG("ui.mini_game.GameUI", GameUI);
        })(mini_game = ui.mini_game || (ui.mini_game = {}));
    })(ui || (ui = {}));

    class GameConstant {
        static getMapRes(name) {
            return `mini_game/27/map_${name}.prefab`;
        }
        static getPrefab(name) {
            return `mini_game/27/${name}.prefab`;
        }
        static getSkelRes(name) {
            return `mini_game/27/ani/${name}.skel`;
        }
        static getEffectRes(name) {
            return `mini_game/27/ani/effect/${name}.skel`;
        }
        static getImageRes(name) {
            return `mini_game/27/game/${name}.png`;
        }
        static getSoundRes(name) {
            return `mini_game/27/sound/${name}.wav`;
        }
        static getBgMusic() {
            let getshi = window["configInfo"] ? "mp3" : "wav";
            return `mini_game/27/sound/bg_game.${getshi} `;
        }
    }
    GameConstant.GUIDE_1 = 1001;
    GameConstant.GUIDE_2 = 1002;
    GameConstant.BLOCK_DEBUG = false;
    GameConstant.AI_TEST = false;
    GameConstant.GUIDE_SWITCH = false;
    GameConstant.GAME_COUNT_DELAY = 120;
    GameConstant.GAME_ID = 27;
    GameConstant.CHECK_VICTORY_TIME = 60 * 1;
    GameConstant.AI_SWITCH = true;
    GameConstant.AI_EXECUTE_SECOND = [10, 11, 12, 13, 14, 15];
    GameConstant.BLOCK_COUNT_MAX = 50;
    GameConstant.ARROW_CIRCLE = 70;
    GameConstant.MAIN_PLAYER_FACTION = "bule";
    GameConstant.pathFix = "";
    GameConstant.BLOCK_NAMES = ["map_1", "map_2", "map_3", "map_4", "map_5", "map_6", "map_7", "map_8", "map_9", "map_10"];
    GameConstant.BLOCK_FACTION = ["bule", "neutral", "red", "yellow", "green"];
    GameConstant.BLOCK_NEIGHBOUR = [
        [0, 2, 5, 4, 0, 0], [0, 0, 3, 5, 1, 0], [0, 0, 0, 6, 5, 2],
        [1, 5, 8, 7, 0, 0], [2, 3, 6, 8, 4, 1], [3, 0, 0, 9, 8, 5],
        [4, 8, 10, 0, 0, 0], [5, 6, 9, 10, 7, 4], [6, 0, 0, 0, 10, 8],
        [8, 9, 0, 0, 0, 7]
    ];
    GameConstant.soldier_temp = {
        '1': { 'type': 1, 'w_h': 16, 'ball_with': 1, 'radius': 8, 'px': 11, 'py': 11, 'res': 'mini_game/27/game/c1.png' },
        '2': { 'type': 2, 'w_h': 20, 'ball_with': 2, 'radius': 10, 'px': 12, 'py': 12, 'res': 'mini_game/27/game/c2.png' },
        '3': { 'type': 3, 'w_h': 24, 'ball_with': 3, 'radius': 12, 'px': 13, 'py': 13, 'res': 'mini_game/27/game/c3.png' }
    };
    GameConstant.FactionResMap = {
        'bule': "lanb",
        'neutral': "minbing",
        'red': "hongb",
        'yellow': "huangb",
        'green': "lvb"
    };
    GameConstant.skel_list = [
        "mini_game/27/ani/hongb1.skel",
        "mini_game/27/ani/hongb2.skel",
        "mini_game/27/ani/hongb3.skel"
    ];
    GameConstant.SOUND_ATTCAK = "attcak";
    GameConstant.SOUND_CONNECT = "connect";
    GameConstant.SOUND_FAILED = "failed";
    GameConstant.SOUND_WIN = "win";
    GameConstant.SOUND_SHOOT = "shoot";
    const GameState = {
        idle: "idle",
        runing: "runing",
        end: "end",
    };
    const GameEffect = {
        baiguan: "shouji",
        gquan: "gquan",
        hongyanwu: "hongyanwu",
        huangyanwu: "huangyanwu",
        lanyanwu: "lanyanwu",
        lvyanwu: "lvyanwu",
        yanwu: "yanwu",
    };
    const BlockAI = {
        easy: "easy",
        simple: "simple",
        heart: "heart",
    };
    const BlockFaction = {
        bule: "bule",
        neutral: "neutral",
        red: "red",
        yellow: "yellow",
        green: "green",
    };
    const SoldierState = {
        attack: "attack",
        run_loop: "run_loop",
        standby_loop: "standby_loop",
        win_loop: "win_loop",
    };
    const MiniGameEvent = {
        BLOCK_GAME_RESULT: "GameEvent.BLOCK_GAME_RESULT",
        SOLDIER_WIN: "GameEvent.SOLDIER_WIN",
        SOLDIER_MOVE: "GameEvent.SOLDIER_MOVE",
        SOLDIER_ATTACK_BLOCK: "GameEvent.SOLDIER_ATTACK_BLOCK",
        SOLDIER_MERGE_BLOCK: "GameEvent.SOLDIER_MERGE_BLOCK",
        SOLDIER_FAKE_ATTACK: "GameEvent.SOLDIER_FAKE_ATTACK",
        SOLDIER_CHNAGE: "GameEvent.SOLDIER_CHANGE",
        SOLDIER_CLEAR_LOCK: "GameEvent.SOLDIER_CLEAR_LOCK",
    };

    class SoundMgr {
        static getInstance() {
            if (!SoundMgr._instance) {
                SoundMgr._instance = new SoundMgr();
            }
            return SoundMgr._instance;
        }
        musicStateUpdate(open) {
            logI(" musicStateUpdate:" + open);
            this._musicOpen = open;
            if (open) {
                this.playBgMusic();
            }
            else {
                this.stopBgMusic();
                this.stopAllSound();
            }
        }
        stopBgMusic() {
            Laya.SoundManager.stopMusic();
        }
        stopAllSound() {
            Laya.SoundManager.stopAllSound();
        }
        playBgMusic() {
            if (!this._musicOpen)
                return;
            let url = GameConstant.getBgMusic();
            if (Laya.loader.getRes(url)) {
                Laya.SoundManager.playMusic(url, 0);
            }
            else {
                logI('reload playBgMusic');
                Laya.loader.load(url, Laya.Handler.create(this, this.playBgMusic));
            }
        }
        playSound(url, loops = 1, complete = null) {
            if (!this._musicOpen)
                return;
            Laya.SoundManager.playSound(GameConstant.getSoundRes(url), loops, complete);
        }
    }

    class GameUtil {
        static tweenPromise(target, props, duration, ease, delay) {
            return new Promise((resolve, reject) => {
                if (!target) {
                    resolve(false);
                    return;
                }
                Laya.Tween.to(target, props, duration, ease, Laya.Handler.create(target, () => {
                    resolve(true);
                }), delay);
            });
        }
        static timerOncePromise(duration, target) {
            return new Promise((resolve, reject) => {
                Laya.timer.once(duration, target, resolve);
            });
        }
        static copyArr2(sourceArr) {
            let result = [];
            for (let arr of sourceArr) {
                let copy = [].concat(arr);
                result.push(copy);
            }
            return result;
        }
        static targetAlpha(targetArr, start, end, duation) {
            let plist = [];
            for (let i = 0; i < targetArr.length; i++) {
                targetArr[i].alpha = start;
                plist.push(GameUtil.tweenPromise(targetArr[i], { alpha: end }, duation));
            }
            return Promise.all(plist);
        }
        static targetVisble(targetArr, visible) {
            for (let i = 0; i < targetArr.length; i++) {
                targetArr[i].visible = visible;
            }
        }
        static tweenJelly(target, posY, delay) {
            target.y = posY;
            target.scale(1, 1);
            let endY = posY - 20;
            Laya.Tween.clearTween(target);
            GameUtil.tweenPromise(target, { scaleY: 0.9 }, 200, Laya.Ease.linearNone).then(() => {
                return GameUtil.tweenPromise(target, { y: endY, scaleY: 1.05 }, 300, Laya.Ease.linearNone);
            }).then(() => {
                return GameUtil.tweenPromise(target, { scaleY: 1 }, 100, Laya.Ease.linearNone);
            }).then(() => {
                return GameUtil.tweenPromise(target, { y: posY, scaleY: 0.9 }, 200, Laya.Ease.linearNone);
            }).then(() => {
                return GameUtil.tweenPromise(target, { scaleY: 1 }, 100, Laya.Ease.linearNone);
            }).then(() => {
                return GameUtil.tweenPromise(target, {}, delay);
            }).then(() => {
                GameUtil.tweenJelly(target, posY, delay);
            });
        }
        static tweenLoopMove(target, x1, y1, x2, y2, duation) {
            Laya.Tween.clearTween(target);
            target.pos(x1, y1);
            target.alpha = 1;
            GameUtil.tweenPromise(target, { x: x2, y: y2 }, duation, Laya.Ease.linearNone).then(() => {
                return GameUtil.tweenPromise(target, { alpha: 0 }, 100, Laya.Ease.linearNone);
            }).then(() => {
                GameUtil.tweenLoopMove(target, x1, y1, x2, y2, duation);
            });
        }
        static getQueryVariable(variable) {
            var query = window.location.search.substring(1);
            var vars = query.split("&");
            for (var i = 0; i < vars.length; i++) {
                var pair = vars[i].split("=");
                if (pair[0] == variable) {
                    return pair[1];
                }
            }
            return "";
        }
        static simpleScaleTween(target, scale1, scale2, duation, isLoop) {
            Laya.Tween.clearTween(target);
            target.scale(scale1, scale1);
            GameUtil.tweenPromise(target, { scaleX: scale2, scaleY: scale2 }, duation).then(() => {
                return GameUtil.tweenPromise(target, { scaleX: scale1, scaleY: scale1 }, duation);
            }).then(() => {
                if (isLoop) {
                    GameUtil.simpleScaleTween(target, scale1, scale2, duation, isLoop);
                }
            });
        }
        static destorySomeSp(spArr) {
            for (let sp of spArr) {
                sp.stop();
                sp.destroy();
            }
        }
        static playSpAnim(spArr, animName, rate, loop) {
            for (let sp of spArr) {
                sp.play(animName, loop);
                sp.playbackRate(rate);
            }
        }
        static playSoundLoop(caller, soundName, delay) {
            SoundMgr.getInstance().playSound(soundName);
            GameUtil.tweenPromise(caller, {}, delay).then(() => {
                GameUtil.playSoundLoop(caller, soundName, delay);
            });
        }
        static bezierTween(caller, target, pointList, duration, ease = Laya.Ease.linearNone, delay = 0, completeFunc) {
            let t = 0 - delay;
            let loop = () => {
                if (null == caller || !target || target.destroyed) {
                    Laya.timer.clear(caller, loop);
                    return;
                }
                if (t > duration) {
                    Laya.timer.clear(caller, loop);
                    completeFunc && completeFunc();
                    return;
                }
                t += 20;
                if (t < 0)
                    return;
                let t1 = ease(t > duration ? duration : t, 0, duration, duration);
                let point = GameUtil.bezierCurvePointByTotal(pointList, t1, duration);
                target.x = point.x;
                target.y = point.y;
            };
            if (null == caller) {
                Laya.timer.clear(caller, loop);
                return;
            }
            Laya.timer.loop(20, caller, loop);
        }
        static bezierCurvePointByTotal(pointList, t, totalT) {
            let n = pointList.length;
            let returnVector = new Laya.Point();
            let pascalTriangle = GameUtil.calculate(n);
            for (let i = 0; i < n; i++) {
                let a = Math.pow(1 - t / totalT, n - 1 - i) * Math.pow(t / totalT, i) * pascalTriangle[i];
                returnVector.x += pointList[i].x * a;
                returnVector.y += pointList[i].y * a;
            }
            return returnVector;
        }
        static calculate(n) {
            let list = [1, 1];
            for (let i = 3; i <= n; i++) {
                let colList = [1];
                for (let j = 1; j < i; j++) {
                    colList[j] = list[j - 1] + (j >= list.length ? 0 : list[j]);
                }
                list = colList;
            }
            return list;
        }
    }

    class MgrBase extends Laya.EventDispatcher {
        constructor() {
            super();
        }
        init(data) {
        }
        initOver(data) {
        }
        dispatchBtnData() {
            if (this.hasListener(MgrBase.GAME_BUTTON_DATA)) {
                this.event(MgrBase.GAME_BUTTON_DATA);
            }
        }
    }
    MgrBase.GAME_BUTTON_DATA = "game_button_data";

    class PoolMgr {
        constructor() {
            this.poolNameObj = {};
        }
        init(data) {
        }
        initOver(data) {
        }
        recover(sign, item) {
            if (!this.poolNameObj[sign]) {
                this.poolNameObj[sign] = sign;
            }
            Laya.Pool.recover(sign, item);
        }
        getItem(sign) {
            return Laya.Pool.getItem(sign);
        }
        clearAll() {
            for (let key in this.poolNameObj) {
                Laya.Pool.clearBySign(key);
            }
            this.poolNameObj = {};
        }
    }

    class BlockData {
        constructor(id, name, faction, count, ai) {
            this.id = 0;
            this.name = null;
            this.neighbour = null;
            this._count = 0;
            this.faction = null;
            this.ai = BlockAI.easy;
            this.id = id;
            this.name = name;
            this.faction = faction;
            this._count = count;
            this.ai = ai;
            this.neighbour = GameConstant.BLOCK_NEIGHBOUR[this.id - 1];
        }
        get count() {
            return this._count;
        }
        changeData(data) {
            logI("被占:" + this.name + " 老阵:" + this.faction + " 老A:" + this.ai + " 发兵:" + data.name + " 新阵:" + data.faction + " 新A:" + data.ai);
            this.faction = data.faction;
            this.ai = data.ai;
        }
        updateCount(value = -1) {
            let lv1 = this.getSoldierLevel();
            if (value != -1) {
                this._count = value;
            }
            else {
                this._count++;
            }
            let lv2 = this.getSoldierLevel();
            return lv1 != lv2;
        }
        beAttacked(attack) {
            if (attack > this.count) {
                let value = attack - (this.count + 1);
                let lv_chnage = this.updateCount(value);
                return { "lv_chnage": lv_chnage, "success": true };
            }
            else {
                let value = this.count - attack;
                let lv_chnage = this.updateCount(value);
                return { "lv_chnage": lv_chnage, "success": false };
            }
        }
        getSoldierLevel() {
            if (this.count < 30) {
                return 1;
            }
            if (this.count < 50) {
                return 2;
            }
            return 3;
        }
        createSoldierData(soldier_id, count, tar, tar_pos) {
            let obj = new SoldierData(this.id, this.name, this.faction, count, this.ai);
            obj.target = tar;
            obj.tar_pos = tar_pos;
            obj.soldier_id = soldier_id;
            obj.attack_ing = false;
            return obj;
        }
    }
    class SoldierData extends BlockData {
        attack() {
            this.attack_ing = true;
        }
        resetAttack() {
            this.attack_ing = false;
        }
        isMerge() {
            return this.faction == this.target.faction;
        }
        updateSoldierCount(value) {
            super._count = value;
        }
    }
    class PlayerAI {
        constructor(faction, ai_second) {
            this.faction = null;
            this.faction_name = null;
            this.ids = null;
            this._rate = 10;
            this._race_count = 1;
            this._test_count = 0;
            this.faction = faction;
            this._ai_second = ai_second;
            this._rate = this.createRate();
            if (faction == BlockFaction.red) {
                this.faction_name = "红";
            }
            if (faction == BlockFaction.yellow) {
                this.faction_name = "黄";
            }
            if (faction == BlockFaction.green) {
                this.faction_name = "绿";
            }
            if (GameConstant.AI_TEST) {
                if (faction == BlockFaction.red) {
                    this._rate = 2;
                }
                if (faction == BlockFaction.yellow) {
                    this._rate = 1;
                }
            }
        }
        isExecuteAITest() {
            if (this._test_count >= 50) {
                return false;
            }
            if (this._race_count % (this._rate * 60) == 0) {
                logI(this.faction_name + "AI 间隔 " + this._rate + "秒");
                this._race_count = 1;
                this._test_count++;
                return true;
            }
            this._race_count++;
            return false;
        }
        isExecuteAI() {
            if (this._race_count % (this._rate * 60) == 0) {
                logI(this.faction_name + "AI 间隔 " + this._rate + "秒");
                this._rate = this.createRate();
                this._race_count = 1;
                return true;
            }
            this._race_count++;
            return false;
        }
        addId(id) {
            this.ids.push(id);
        }
        createRate() {
            var index = Math.round(Math.random() * (this._ai_second.length - 1));
            return this._ai_second[index];
        }
        randomId() {
            if (this.existIds()) {
                if (this.ids.length == 1) {
                    return this.ids[0];
                }
                var r = Math.random() * this.ids.length;
                return this.ids[r];
            }
            return -1;
        }
        delId(id) {
            var index = this.ids.indexOf(id);
            if (index > -1) {
                this.ids.splice(index, 1);
            }
        }
        existIds() {
            return this.ids && this.ids.length > 0;
        }
    }

    class MapMgr {
        constructor() {
            this._soldiers = new Map();
            this._blocks = new Map();
            this.players = [];
        }
        get game_state() {
            return this._game_state;
        }
        gameStart() {
            this._game_state = GameState.runing;
        }
        gameEnd() {
            this._game_state = GameState.end;
        }
        gameIsRuning() {
            return this._game_state == GameState.runing;
        }
        init() {
        }
        initOver(data) {
        }
        initData(curCfg) {
            this._game_state = GameState.idle;
            this.players = [];
            this.players.push(new PlayerAI(BlockFaction.red, curCfg.ai_second));
            this.players.push(new PlayerAI(BlockFaction.yellow, curCfg.ai_second));
            this.players.push(new PlayerAI(BlockFaction.green, curCfg.ai_second));
            this._blocks.clear();
            this._soldiers.clear();
            this.curCfg = curCfg;
        }
        addBlock(block) {
            if (this._blocks.get(block.id)) {
                logE("Block 块数据已存在" + block.id);
                return;
            }
            this._blocks.set(block.id, block);
        }
        getBlock(id) {
            return this._blocks.get(id);
        }
        getBlockByFaction(faction) {
            let datas = [];
            this._blocks.forEach((value, key) => {
                if (value.faction == faction) {
                    datas.push(value);
                }
            });
            return datas;
        }
        getBlockEnemy(faction) {
            let datas = [];
            this._blocks.forEach((value, key) => {
                if (value.faction != faction && value.faction != BlockFaction.neutral) {
                    datas.push(value);
                }
            });
            return datas;
        }
        getBlockByFactionMap(faction) {
            let map = new Map();
            this._blocks.forEach((value, key) => {
                if (value.faction == faction) {
                    map.set(key, value);
                }
            });
            return map;
        }
        getBlockAll() {
            let map = new Map();
            this._blocks.forEach((value, key) => {
                map.set(key, value);
            });
            return map;
        }
        delBlock(id) {
            let bd = this._blocks.get(id);
            if (bd) {
                this._blocks.delete(id);
            }
        }
        getBlockIds(faction) {
            let ids = [];
            this._blocks.forEach((value, key) => {
                if (value.faction == faction && value.count > 0) {
                    ids.push(key);
                }
            });
            return ids;
        }
        isSoldiers(block_id) {
            return this._soldiers.get(block_id);
        }
        clearAll() {
            this._soldiers.clear();
            this._blocks.clear();
        }
    }

    class DataMgr {
        static get evetMgr() { return DataMgr._evntMgr; }
        static get poolMgr() { return DataMgr._poolMgr; }
        static get mapMgr() { return DataMgr._mapMgr; }
        static init() {
            DataMgr._evntMgr = new MgrBase;
            DataMgr._evntMgr.init();
            DataMgr._mapMgr = new MapMgr;
            DataMgr._mapMgr.init();
            DataMgr._poolMgr = new PoolMgr;
            DataMgr._poolMgr.init();
        }
        static initOver() {
            if (DataMgr.LoginOver)
                return;
            DataMgr.LoginOver = true;
            DataMgr._evntMgr.initOver();
            DataMgr._mapMgr.initOver();
            DataMgr._poolMgr.initOver();
        }
    }
    DataMgr.LoginOver = false;

    class TweenEx {
        static to(target, props, duration, ease = null) {
            return new Promise((res, rej) => {
                Laya.Tween.to(target, props, duration, ease, Laya.Handler.create(this, () => {
                    res('tween done');
                }));
            });
        }
    }
    class misc {
        static clampf(value, min_inclusive, max_inclusive) {
            if (min_inclusive > max_inclusive) {
                var temp = min_inclusive;
                min_inclusive = max_inclusive;
                max_inclusive = temp;
            }
            return value < min_inclusive ? min_inclusive : value < max_inclusive ? value : max_inclusive;
        }
        ;
        static radiansToDegrees(angle) {
            return angle * 180 / Math.PI;
        }
        static lineLine(a1, a2, b1, b2) {
            var ua_t = (b2.x - b1.x) * (a1.y - b1.y) - (b2.y - b1.y) * (a1.x - b1.x);
            var ub_t = (a2.x - a1.x) * (a1.y - b1.y) - (a2.y - a1.y) * (a1.x - b1.x);
            var u_b = (b2.y - b1.y) * (a2.x - a1.x) - (b2.x - b1.x) * (a2.y - a1.y);
            if (u_b !== 0) {
                var ua = ua_t / u_b;
                var ub = ub_t / u_b;
                if (0 <= ua && ua <= 1 && 0 <= ub && ub <= 1) {
                    return true;
                }
            }
            return false;
        }
        static lineLinePoint(a, b, c, d) {
            let area_abc = (a.x - c.x) * (b.y - c.y) - (a.y - c.y) * (b.x - c.x);
            let area_abd = (a.x - d.x) * (b.y - d.y) - (a.y - d.y) * (b.x - d.x);
            if (area_abc * area_abd >= 0) {
                return null;
            }
            let area_cda = (c.x - a.x) * (d.y - a.y) - (c.y - a.y) * (d.x - a.x);
            let area_cdb = area_cda + area_abc - area_abd;
            if (area_cda * area_cdb >= 0) {
                return null;
            }
            let t = area_cda / (area_abd - area_abc);
            let dx = t * (b.x - a.x), dy = t * (b.y - a.y);
            return { x: a.x + dx, y: a.y + dy };
        }
        static lineCircle(a, b, c) {
            let sd = a.sub(c.position).mag();
            let ed = b.sub(c.position).mag();
            if (sd === c.radius || ed === c.radius) {
                return true;
            }
            if (sd < c.radius && ed < c.radius) {
                return false;
            }
            if ((sd < c.radius && ed > c.radius) || (sd > c.radius && ed < c.radius)) {
                return true;
            }
            let rd = this.pointLineDistance(c.position, a, b);
            if (rd === c.radius) {
                return true;
            }
            return false;
        }
        static pointLineDistance(point, start, end, isSegment = true) {
            var dx = end.x - start.x;
            var dy = end.y - start.y;
            var d = dx * dx + dy * dy;
            var t = ((point.x - start.x) * dx + (point.y - start.y) * dy) / d;
            var p;
            if (!isSegment) {
                p = new Vec2(start.x + t * dx, start.y + t * dy);
            }
            else {
                if (d) {
                    if (t < 0)
                        p = start;
                    else if (t > 1)
                        p = end;
                    else
                        p = new Vec2(start.x + t * dx, start.y + t * dy);
                }
                else {
                    p = start;
                }
            }
            dx = point.x - p.x;
            dy = point.y - p.y;
            return Math.sqrt(dx * dx + dy * dy);
        }
        static circleCircle(a, b) {
            var distance = a.position.sub(b.position).mag();
            return distance < (a.radius + b.radius);
        }
        static pointInLine(p, s, e) {
            if ((p.x - s.x) * (e.y - s.y) == (e.x - s.x) * (p.y - s.y)
                && Math.min(s.x, e.x) <= p.x && p.x <= Math.max(s.x, e.x)
                && Math.min(s.y, e.y) <= p.y && p.y <= Math.max(s.y, e.y)) {
                return true;
            }
            else {
                return false;
            }
        }
        static lineRect(a1, a2, b) {
            let r0 = new Vec2(b.x, b.y);
            let r1 = new Vec2(b.x, b.y + b.height);
            let r2 = new Vec2(b.x + b.width, b.y + b.height);
            let r3 = new Vec2(b.x + b.width, b.y);
            if (this.lineLine(a1, a2, r0, r1)) {
                return true;
            }
            if (this.lineLine(a1, a2, r1, r2)) {
                return true;
            }
            if (this.lineLine(a1, a2, r2, r3)) {
                return true;
            }
            if (this.lineLine(a1, a2, r3, r0)) {
                return true;
            }
            return false;
        }
        static parseParamFromUrl() {
            let query = location.search && (location.search.substring(1) || "");
            if (!query) {
                return {};
            }
            let queryArr = query.split("&");
            let result = queryArr.reduce((pre, ele) => {
                let key = ele.split("=")[0];
                let value = ele.split("=")[1];
                pre[key] = value;
                return pre;
            }, Object.create(null));
            return result;
        }
    }
    misc.EPS = 1e-10;
    class Vec2 extends Laya.Point {
        static get ZERO() { return new Vec2(0, 0); }
        ;
        static get RIGHT() { return new Vec2(1, 0); }
        ;
        static get LEFT() { return new Vec2(-1, 0); }
        ;
        static get UP() { return new Vec2(0, 1); }
        ;
        static get DOWN() { return new Vec2(0, -1); }
        ;
        static subtract(out, a, b) {
            out.x = a.x - b.x;
            out.y = a.y - b.y;
            return out;
        }
        static multiply(out, a, b) {
            out.x = a.x * b.x;
            out.y = a.y * b.y;
            return out;
        }
        constructor(x = 0, y = 0) {
            super();
            if (x && typeof x === 'object') {
                this.x = x.x || 0;
                this.y = x.y || 0;
            }
            else {
                this.x = x || 0;
                this.y = y || 0;
            }
        }
        clone() {
            return new Vec2(this.x, this.y);
        }
        scale(vector, out) {
            return Vec2.multiply(out || new Vec2(), this, vector);
        }
        add(vector, out) {
            out = out || new Vec2();
            out.x = this.x + vector.x;
            out.y = this.y + vector.y;
            return out;
        }
        addSelf(vector) {
            this.x += vector.x;
            this.y += vector.y;
            return this;
        }
        sub(vector, out) {
            return Vec2.subtract(out || new Vec2(), this, vector);
        }
        subSelf(vector) {
            this.x -= vector.x;
            this.y -= vector.y;
            return this;
        }
        mag() {
            return Math.sqrt(this.x * this.x + this.y * this.y);
        }
        magSqr() {
            return this.x * this.x + this.y * this.y;
        }
        dot(vector) {
            return this.x * vector.x + this.y * vector.y;
        }
        cross(vector) {
            return this.x * vector.y - this.y * vector.x;
        }
        normalizeSelf() {
            var magSqr = this.x * this.x + this.y * this.y;
            if (magSqr === 1.0)
                return this;
            if (magSqr === 0.0) {
                return this;
            }
            var invsqrt = 1.0 / Math.sqrt(magSqr);
            this.x *= invsqrt;
            this.y *= invsqrt;
            return this;
        }
        normalize(out) {
            out = out || new Vec2();
            out.x = this.x;
            out.y = this.y;
            out.normalizeSelf();
            return out;
        }
        angle(vector) {
            var magSqr1 = this.magSqr();
            var magSqr2 = vector.magSqr();
            if (magSqr1 === 0 || magSqr2 === 0) {
                logW("Can't get angle between zero vector");
                return 0.0;
            }
            var dot = this.dot(vector);
            var theta = dot / (Math.sqrt(magSqr1 * magSqr2));
            theta = misc.clampf(theta, -1.0, 1.0);
            return Math.acos(theta);
        }
        signAngle(vector) {
            let angle = this.angle(vector);
            return this.cross(vector) < 0 ? -angle : angle;
        }
    }
    window["Vec2"] = Vec2;

    class MapArrow extends Laya.Script {
        constructor() {
            super();
        }
        onEnable() {
            this._circle = this.owner.getChildByName("circle");
            this._arrow = this.owner.getChildByName("arrow");
            this._init_pos = this.owner.getChildByName("init_pos");
            this._focus = this.owner.getChildByName("focus");
        }
        createPoint(parent) {
            return this._init_pos.localToGlobal(new Laya.Point(0, 0), true, parent);
        }
        showArrow() {
            this._arrow.visible = true;
            this._circle.visible = true;
        }
        showFocus(my_faction) {
            let res_name = my_faction ? "white_circle" : "red_circle";
            let url = GameConstant.getImageRes(res_name);
            this._focus.loadImage(url);
            this._focus.visible = true;
            SoundMgr.getInstance().playSound(GameConstant.SOUND_CONNECT);
        }
        isFocus() {
            return this._focus.visible;
        }
        hideFocus() {
            this._focus.visible = false;
        }
        updateArrow(nx, ny) {
            var p1 = new Laya.Point(nx, ny);
            var p2 = this.createPoint(Laya.stage);
            let ropeVec = new Vec2(p2.x - p1.x, p2.y - p1.y);
            let len = ropeVec.mag();
            if (len < 10) {
                len = 0;
            }
            let len1 = len * 0.01;
            this._arrow.scaleX = len1;
            let angle = misc.radiansToDegrees(Vec2.RIGHT.signAngle(ropeVec));
            this._arrow.rotation = angle;
        }
        hideArrow() {
            this._arrow.visible = false;
            this._circle.visible = false;
        }
        onDisable() {
        }
    }

    class Soldier extends Laya.Script {
        constructor() {
            super(...arguments);
            this._face_right = true;
            this._state = SoldierState.standby_loop;
        }
        onEnable() {
            DataMgr.evetMgr.on(MiniGameEvent.SOLDIER_FAKE_ATTACK, this, this.onFakeAttack);
            DataMgr.evetMgr.on(MiniGameEvent.SOLDIER_WIN, this, this.onWinloop);
        }
        init(block, parent, pos) {
            this._owner = this.owner;
            this._block = block;
            this._pos = pos;
            parent.addChild(this._owner);
            this._count = this.owner.getChildByName("count");
            this._owner.x = pos.x;
            this._owner.y = pos.y;
            this.onCreateAni();
            if (this.isSoldierData()) {
                this.updateCount();
            }
        }
        updateCount() {
            this._count.visible = true;
            this._count.text = this._block.count + "";
        }
        playWuEffect() {
            let uri = "";
            if (this._block.faction == BlockFaction.bule) {
                uri = GameEffect.lanyanwu;
            }
            if (this._block.faction == BlockFaction.yellow) {
                uri = GameEffect.huangyanwu;
            }
            if (this._block.faction == BlockFaction.green) {
                uri = GameEffect.lvyanwu;
            }
            if (this._block.faction == BlockFaction.red) {
                uri = GameEffect.hongyanwu;
            }
            if (!this._wu_sp) {
                this._wu_sp = inner.MiniGameSpineMgr.createSpine(GameConstant.GAME_ID, GameConstant.getEffectRes(uri));
            }
            this._wu_sp.stop();
            this._wu_sp.playByIndex(0, false);
            this._owner.addChild(this._wu_sp);
        }
        playWhiteEffect() {
            if (!this._white_sp) {
                this._white_sp = inner.MiniGameSpineMgr.createSpine(GameConstant.GAME_ID, GameConstant.getEffectRes(GameEffect.baiguan));
            }
            this._white_sp.stop();
            this._white_sp.playByIndex(0, false);
            this._owner.addChild(this._white_sp);
            this._white_sp.pos((this._owner.width / 2), (this._owner.height / 2) - 80);
        }
        onCreateAni() {
            let suffix = GameConstant.FactionResMap[this._block.faction];
            let sk_uri = null;
            if (this._block.faction == BlockFaction.neutral) {
                sk_uri = GameConstant.getSkelRes("minbing");
            }
            else {
                let lv = this._block.getSoldierLevel();
                sk_uri = GameConstant.getSkelRes(suffix + lv);
            }
            this._ani = inner.MiniGameSpineMgr.createSpine(GameConstant.GAME_ID, sk_uri);
            this._state = SoldierState.standby_loop;
            this.updateFace();
            GameUtil.playSpAnim([this._ani], this._state, 1, true);
            this._owner.addChildAt(this._ani, 0);
        }
        setState(state) {
            if (this._block.faction == BlockFaction.neutral && state != SoldierState.standby_loop) {
                return;
            }
            this._state = state;
            GameUtil.playSpAnim([this._ani], this._state, 1, true);
        }
        getState() {
            return this._state;
        }
        isSoldierData() {
            return this._block instanceof SoldierData;
        }
        getSoldierData() {
            if (this._block instanceof SoldierData) {
                return this._block;
            }
            logE("没找到士兵数据.");
            return null;
        }
        onSoldierChange() {
            this._ani.destroy();
            this._owner.removeChild(this._ani);
            this._ani = null;
            this.onCreateAni();
        }
        getOwner() {
            return this._owner;
        }
        moveStart() {
            this.setState(SoldierState.run_loop);
        }
        moveComplete() {
            let cur = this.getSoldierData();
            if (cur.isMerge()) {
                this.setState(SoldierState.standby_loop);
                DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_MERGE_BLOCK, [cur]);
                this.owner.removeSelf();
                return;
            }
            else {
                this.attackStart();
            }
        }
        attackStart() {
            this.setState(SoldierState.attack);
            SoundMgr.getInstance().playSound(GameConstant.SOUND_ATTCAK);
            Laya.timer.once(700, this, this.attackComplete);
            let data = this.getSoldierData();
            let x = this._owner.x;
            let y = this._owner.y;
            this.playWhiteEffect();
            DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_FAKE_ATTACK, [data.target.id, x, y]);
        }
        attackComplete() {
            this.setState(SoldierState.standby_loop);
            this.playWhiteEffect();
            let cur = this.getSoldierData();
            let tar = this.getSoldierData().target;
            DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_ATTACK_BLOCK, [cur, tar]);
            this.owner.removeSelf();
        }
        moveAttackStart() {
            this.setState(SoldierState.attack);
            this.playWhiteEffect();
            SoundMgr.getInstance().playSound(GameConstant.SOUND_ATTCAK);
            Laya.timer.once(700, this, this.moveAttackComplete);
        }
        moveAttackComplete() {
            this.setState(SoldierState.standby_loop);
        }
        removeSoldierSelf() {
            this.owner.removeSelf();
        }
        setFace(is_right) {
            this._face_right = is_right;
            this.updateFace();
        }
        updateFace() {
            if (this._ani) {
                this._ani.scaleX = this._face_right ? 1 : -1;
            }
        }
        onFakeAttack(block_id, x, y) {
            if (this._block.id != block_id) {
                return;
            }
            if (this.isSoldierData()) {
                return;
            }
            if (x > this._owner.x) {
                this.setFace(true);
            }
            else {
                this.setFace(false);
            }
            this.setState(SoldierState.attack);
            GameUtil.playSpAnim([this._ani], this._state, 1, true);
            Laya.timer.once(700, this, this.moveAttackComplete);
            this.playWhiteEffect();
        }
        fakeAttackComplete() {
            this.setState(SoldierState.standby_loop);
            GameUtil.playSpAnim([this._ani], this._state, 1, true);
        }
        onWinloop(is_victory) {
            let is_my = this._block.faction == GameConstant.MAIN_PLAYER_FACTION;
            if (is_victory) {
                if (is_my) {
                    this.setState(SoldierState.win_loop);
                }
            }
            else {
                if (!is_my) {
                    this.setState(SoldierState.win_loop);
                }
            }
        }
        onUpdate() {
        }
        onDisable() {
            DataMgr.evetMgr.off(MiniGameEvent.SOLDIER_FAKE_ATTACK, this, this.onFakeAttack);
            DataMgr.evetMgr.off(MiniGameEvent.SOLDIER_WIN, this, this.onWinloop);
        }
        destorySpine() {
            if (this._ani) {
                this._ani.destroy();
                this._ani.removeSelf();
                this._ani = null;
            }
            if (this._white_sp) {
                this._white_sp.destroy();
                this._white_sp.removeSelf();
                this._white_sp = null;
            }
            if (this._wu_sp) {
                this._wu_sp.destroy();
                this._wu_sp.removeSelf();
                this._wu_sp = null;
            }
        }
        onDestroy() {
            this.destorySpine();
        }
    }

    class MapScript extends Laya.Script {
        constructor() {
            super();
            this._block_s = new Map();
            this._move_soldier_id = 1;
            this._move_soldier = new Map();
            this._lock_soldier = new Map();
            this._arrows = new Map();
            this._race_count = 0;
            this._sendSoldierAI = false;
            this._arrow_flag = new Map();
            this._focus = null;
            this._guide_ing = false;
        }
        onEnable() {
            this._owner = this.owner;
            for (let i = 1; i <= 10; i++) {
                let map_arrow = this.owner.getChildByName("arrow_" + i);
                if (map_arrow) {
                    let map_arrow_s = map_arrow.getComponent(MapArrow);
                    this._arrows.set(i, map_arrow_s);
                }
            }
            DataMgr.evetMgr.on(MiniGameEvent.SOLDIER_CLEAR_LOCK, this, this.clearLockList);
        }
        init(curCfg, soldier_prefab) {
            this._curCfg = curCfg;
            this._soldier_prefab = soldier_prefab;
            this._tips = this.owner.getChildByName("tips");
        }
        addMapBlock(id, block_s) {
            this._block_s.set(id, block_s);
        }
        onUpdate() {
            if (!DataMgr.mapMgr.gameIsRuning()) {
                return;
            }
            this._race_count++;
            if (GameConstant.AI_SWITCH) {
                for (let i = 0; i < DataMgr.mapMgr.players.length; i++) {
                    let player = DataMgr.mapMgr.players[i];
                    if (GameConstant.AI_TEST) {
                        if (player.isExecuteAITest()) {
                            this.executeAI(player);
                        }
                    }
                    else {
                        if (player.isExecuteAI()) {
                            this.executeAI(player);
                        }
                    }
                }
            }
            this.checkMoveTrigger();
            this.checkVictory();
        }
        checkVictory() {
            if (GameConstant.AI_TEST) {
                return;
            }
            if (this._race_count % GameConstant.CHECK_VICTORY_TIME == 0) {
                let has_block = 0;
                let has_soldier = 0;
                let neutral_count = 0;
                this._block_s.forEach((value, key) => {
                    let data = value.getBlock();
                    if (data.faction == GameConstant.MAIN_PLAYER_FACTION) {
                        has_block++;
                    }
                    if (data.faction == BlockFaction.neutral) {
                        neutral_count++;
                    }
                });
                this._move_soldier.forEach((move_data, key) => {
                    if (move_data.cur_faction == GameConstant.MAIN_PLAYER_FACTION) {
                        has_soldier++;
                    }
                });
                let lock_soldier = 0;
                let my_lock_soldier = 0;
                this._lock_soldier.forEach((move_data, key) => {
                    if (move_data.cur_faction != GameConstant.MAIN_PLAYER_FACTION) {
                        lock_soldier++;
                    }
                    else {
                        my_lock_soldier++;
                    }
                });
                if (lock_soldier > 0) {
                    return;
                }
                if (has_block <= 0 && has_soldier <= 0 && my_lock_soldier <= 0) {
                    this.gameResult(false);
                    return;
                }
                let block_victory = (has_block == this._block_s.size) || (this._block_s.size - neutral_count == has_block);
                if (block_victory && has_soldier == this._move_soldier.size) {
                    this.gameResult(true);
                    return;
                }
            }
        }
        gameResult(is_victory) {
            DataMgr.mapMgr.gameEnd();
            this._move_soldier.forEach((move_data, key) => {
                if (move_data && move_data.tween) {
                    move_data.tween.pause();
                    move_data.tween = null;
                    move_data.soldier = null;
                }
            });
            this.hideArrows();
            this.hideFocus();
            if (is_victory) {
                SoundMgr.getInstance().playSound(GameConstant.SOUND_WIN);
            }
            else {
                SoundMgr.getInstance().playSound(GameConstant.SOUND_FAILED);
            }
            DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_WIN, [is_victory]);
            Laya.timer.once(2000, null, () => {
                DataMgr.evetMgr.event(MiniGameEvent.BLOCK_GAME_RESULT, [is_victory]);
            });
        }
        executeAI(player) {
            let ids = DataMgr.mapMgr.getBlockIds(player.faction);
            if (ids && ids.length <= 0) {
                return;
            }
            let friend_ids = [];
            if (ids && ids.length === 1) {
                this.onTriggerAI(ids[0], friend_ids);
            }
            if (ids && ids.length > 1) {
                let temp = Math.round(Math.random() * (ids.length - 1));
                let block_id = ids[temp];
                var index = ids.indexOf(block_id);
                if (index > -1) {
                    ids.splice(index, 1);
                }
                friend_ids = ids;
                this.onTriggerAI(block_id, friend_ids);
            }
        }
        onTriggerAI(block_id, friend_ids) {
            let cur = DataMgr.mapMgr.getBlock(block_id);
            if (cur.ai == BlockAI.easy) {
                logI("简单: 不发兵 ");
                return;
            }
            if (cur.ai == BlockAI.simple) {
                logI("中等: ->中立->友方");
                if (this.trySendFeutral(cur)) {
                    return;
                }
                if (this.trySendFriend(cur, friend_ids)) {
                    return;
                }
            }
            if (cur.ai == BlockAI.heart) {
                logI("困难: ->中立->敌方->友方");
                if (this.trySendFeutral(cur)) {
                    return;
                }
                if (this.trySendEnemy(cur)) {
                    return;
                }
                if (this.trySendFriend(cur, friend_ids)) {
                    return;
                }
            }
        }
        trySendFeutral(cur) {
            let neutrals = DataMgr.mapMgr.getBlockByFaction(BlockFaction.neutral);
            if (neutrals && neutrals.length > 0) {
                let tar = this.randomBlockData(neutrals);
                logI("to 中立");
                this.sendSoldier(cur, tar);
                return true;
            }
            return false;
        }
        trySendFriend(cur, friend_ids) {
            if (friend_ids && friend_ids.length > 0) {
                let id = this.randomId(friend_ids);
                let target = DataMgr.mapMgr.getBlock(id);
                logI("to 友方");
                this.sendSoldier(cur, target);
                return true;
            }
            return false;
        }
        trySendEnemy(cur) {
            let enemys = DataMgr.mapMgr.getBlockEnemy(cur.faction);
            if (enemys && enemys.length > 0) {
                let tar = this.randomBlockData(enemys);
                logI("to 敌方");
                this.sendSoldier(cur, tar);
                return true;
            }
            return false;
        }
        randomBlockData(blocks) {
            let index = Math.round(Math.random() * (blocks.length - 1));
            return blocks[index];
        }
        randomId(blocks) {
            let index = Math.round(Math.random() * (blocks.length - 1));
            return blocks[index];
        }
        generateSoldierId() {
            this._move_soldier_id++;
            return this._move_soldier_id;
        }
        sendSoldier(cur, tar) {
            logI("发兵:" + cur.name + " --> 目标" + tar.name);
            let cur_s = this.getMapBlock(cur.id);
            let tar_s = this.getMapBlock(tar.id);
            let takeOut = cur_s.takeOutSoldier();
            let cur_pos = cur_s.createPoint(this._owner);
            let tar_pos = null;
            if (cur.faction == tar.faction) {
                tar_pos = tar_s.createPoint(this._owner);
            }
            else {
                tar_pos = tar_s.createAroundPoint(cur_pos);
            }
            let data = cur.createSoldierData(this.generateSoldierId(), takeOut, tar, tar_pos);
            let soldier = this.createSoldier(data, this._owner, cur_pos);
            let tween = this.moveSoldier(soldier);
            cur_s.playEffectCircle();
            this.addMoveList(soldier, tween, tar);
        }
        moveSoldier(soldier) {
            soldier.moveStart();
            let data = soldier.getSoldierData();
            let sp = soldier.owner;
            if (sp.x > data.tar_pos.x) {
                soldier.setFace(false);
            }
            var point = new Laya.Point(sp.x, sp.y);
            var distance = point.distance(data.tar_pos.x, data.tar_pos.y);
            let duration = distance * 7;
            let tween = Laya.Tween.to(soldier.owner, { x: data.tar_pos.x, y: data.tar_pos.y }, duration, Laya.Ease.linearNone, Laya.Handler.create(this, () => {
                let data = soldier.getSoldierData();
                if (data) {
                    this.delMoveList(data.soldier_id, true);
                }
                soldier.moveComplete();
            }));
            return tween;
        }
        addLockList(soldier_id, move) {
            this._lock_soldier.set(soldier_id, move);
        }
        clearLockList(soldier) {
            let lock_data = this._lock_soldier.get(soldier.soldier_id);
            if (!lock_data) {
                logE("未找到锁定数据 soldier_id:" + soldier.soldier_id);
                return;
            }
            logI("删除锁定:" + soldier.soldier_id);
            this._lock_soldier.delete(soldier.soldier_id);
            lock_data.soldier = null;
        }
        addMoveList(soldier, tween, tar) {
            let data = soldier.getSoldierData();
            this._move_soldier.set(data.soldier_id, new MoveData(soldier, tween, tar.faction));
        }
        delMoveList(soldier_id, is_lock = false) {
            let move_data = this._move_soldier.get(soldier_id);
            if (!move_data) {
                logE("未找到移动数据 soldier_id:" + soldier_id);
                return;
            }
            this._move_soldier.delete(soldier_id);
            move_data.tween.clear();
            move_data.tween = null;
            if (is_lock) {
                this.addLockList(soldier_id, move_data);
                logI("进入锁定:" + soldier_id);
            }
            else {
                move_data.soldier = null;
            }
        }
        checkMoveTrigger() {
            let a_list = [];
            let b_list = [];
            this._move_soldier.forEach((a, key) => {
                this._move_soldier.forEach((b, key) => {
                    let is_faction = a.cur_faction == b.cur_faction;
                    let a_b = a.tar_faction == b.cur_faction;
                    let b_a = b.tar_faction == a.cur_faction;
                    if (!a.trigger && !b.trigger && !is_faction && a_b && b_a) {
                        let a1 = a.soldier.getSoldierData();
                        let b2 = b.soldier.getSoldierData();
                        let attack_ing = a1.attack_ing || b2.attack_ing;
                        let a_sp = a.soldier.getOwner();
                        let b_sp = b.soldier.getOwner();
                        var p1 = new Laya.Point(a_sp.x, a_sp.y);
                        var p2 = new Laya.Point(b_sp.x, b_sp.y);
                        var distance = p1.distance(p2.x, p2.y);
                        if (!attack_ing && distance < 50) {
                            logE("碰撞上了." + a1.soldier_id + "--->" + b2.soldier_id + " attack_ing:" + attack_ing);
                            a_list.push(a);
                            b_list.push(b);
                            a.trigger = true;
                            b.trigger = true;
                        }
                    }
                });
            });
            if (a_list.length <= 0) {
                return;
            }
            this.executeAttackList(a_list, b_list);
        }
        executeAttackList(a_list, b_list) {
            logI("检测到 相互攻击 ");
            for (let i = 0; i < a_list.length; i++) {
                let a = a_list[i];
                let b = b_list[i];
                let a1 = a.soldier.getSoldierData();
                let b2 = b.soldier.getSoldierData();
                a1.attack();
                b2.attack();
                a.tween.pause();
                b.tween.pause();
                a.soldier.moveAttackStart();
                b.soldier.moveAttackStart();
                Laya.timer.once(1000, null, (ma, mb) => {
                    let s1 = ma.soldier.getSoldierData();
                    let s2 = mb.soldier.getSoldierData();
                    s1.resetAttack();
                    s2.resetAttack();
                    logI("a:" + s1.count + "  b:" + s2.count);
                    if (s1.count == s2.count) {
                        logI("平手-----------");
                        ma.soldier.removeSoldierSelf();
                        mb.soldier.removeSoldierSelf();
                        this.delMoveList(s1.soldier_id);
                        this.delMoveList(s2.soldier_id);
                        return;
                    }
                    if (s1.count > s2.count) {
                        let value = s1.count - s2.count;
                        logI("胜利:" + s1.soldier_id);
                        logI("更新前count:" + s1.count);
                        s1.updateCount(value);
                        logI("更新后count:" + s1.count);
                        ma.soldier.onSoldierChange();
                        ma.soldier.updateCount();
                        ma.soldier.moveStart();
                        ma.trigger = false;
                        ma.tween.resume();
                        mb.soldier.removeSoldierSelf();
                        this.delMoveList(s2.soldier_id);
                    }
                    else {
                        logI("胜利:" + s2.soldier_id);
                        let value = s2.count - s1.count;
                        logI("更新前count:" + s2.count);
                        s2.updateCount(value);
                        logI("更新后count:" + s2.count);
                        mb.soldier.onSoldierChange();
                        mb.soldier.updateCount();
                        mb.soldier.moveStart();
                        mb.trigger = false;
                        mb.tween.resume();
                        ma.soldier.removeSoldierSelf();
                        this.delMoveList(s1.soldier_id);
                    }
                }, [a, b]);
            }
        }
        createSoldier(data, parent, cur_pos) {
            let soldier_sp = Laya.Pool.getItemByCreateFun("soldier", this._soldier_prefab.create, this._soldier_prefab);
            let soldier_s = soldier_sp.getComponent(Soldier);
            cur_pos.x = cur_pos.x + 20;
            soldier_s.init(data, parent, cur_pos);
            this._owner.addChild(soldier_sp);
            return soldier_s;
        }
        checkTriggerArrow(nx, ny) {
            this._block_s.forEach((value, key) => {
                var p1 = new Laya.Point(nx, ny);
                var p2 = value.createPoint(Laya.stage);
                var distance = p1.distance(p2.x, p2.y);
                let has_count = value.getBlock().count > 0;
                if (distance < GameConstant.ARROW_CIRCLE && has_count) {
                    let is_flag = this._arrow_flag.get(key) != null;
                    let is_my_faction = value.isFaction(GameConstant.MAIN_PLAYER_FACTION);
                    let arrow = this._arrows.get(value.getBlockId());
                    if (!is_flag && is_my_faction) {
                        this._arrow_flag.set(key, value);
                        arrow.showArrow();
                    }
                    this.showFocus(arrow, is_my_faction);
                }
            });
        }
        showFocus(arrow, is_my_faction) {
            if (this._focus == arrow && this._focus.isFocus()) {
                return;
            }
            this.hideFocus();
            this._focus = arrow;
            this._focus.showFocus(is_my_faction);
        }
        hideFocus() {
            if (this._focus) {
                this._focus.hideFocus();
            }
        }
        checkArrowAttack(nx, ny) {
            this._block_s.forEach((value, key) => {
                let size = this._arrow_flag.size;
                if (size > 0) {
                    var p1 = new Laya.Point(nx, ny);
                    var p2 = value.createPoint(Laya.stage);
                    var distance = p1.distance(p2.x, p2.y);
                    if (distance < GameConstant.ARROW_CIRCLE) {
                        let isMyFaction = value.isFaction(GameConstant.MAIN_PLAYER_FACTION);
                        if (isMyFaction) {
                            let last = this._arrow_flag.get(value.getBlockId());
                            if (last) {
                                this.hideArrow(last);
                                this._arrow_flag.delete(value.getBlockId());
                            }
                        }
                        logI("产生碰撞 1 distance:" + distance);
                        this.sendArrowAttack(value);
                    }
                }
            });
            this.hideFocus();
            this.hideArrows();
        }
        sendArrowAttack(target) {
            this._arrow_flag.forEach((value, key) => {
                let cur = value.getBlock();
                if (cur.faction == GameConstant.MAIN_PLAYER_FACTION) {
                    let tar = target.getBlock();
                    this.sendSoldier(cur, tar);
                }
            });
        }
        hideArrows() {
            this._arrow_flag.forEach((value, key) => {
                this.hideArrow(value);
            });
            this._arrow_flag.clear();
        }
        hideArrow(value) {
            let arrow = this._arrows.get(value.getBlockId());
            arrow.hideArrow();
        }
        updateArrows(nx, ny) {
            this._arrow_flag.forEach((value, key) => {
                let arrow = this._arrows.get(value.getBlockId());
                arrow.updateArrow(nx, ny);
            });
        }
        getMapBlock(id) {
            return this._block_s.get(id);
        }
        onDisable() {
            DataMgr.evetMgr.off(MiniGameEvent.SOLDIER_CLEAR_LOCK, this, this.clearLockList);
            this._block_s.clear();
            this._move_soldier.clear();
            this._lock_soldier.clear();
            this._arrows.clear();
        }
        stopGuide() {
            if (this._guide_tween1) {
                this._guide_tween1.clear();
            }
            if (this._guide_tween2) {
                this._guide_tween2.clear();
            }
            if (this._tips) {
                this._tips.visible = false;
                Laya.Tween.clearTween(this._tips);
            }
            if (this._guide2_p1) {
                Laya.Tween.clearTween(this._guide2_p1);
            }
            this._guide_ing = false;
        }
        playGuide1() {
            this._guide_ing = true;
            let a = this.getMapBlock(8);
            let b = this.getMapBlock(5);
            let p1 = a.createPoint(Laya.stage);
            let p2 = b.createPoint(Laya.stage);
            this._guide_point = new Laya.Point(p1.x, p1.y);
            if (this._guide_tween1) {
                this._guide_tween1.clear();
            }
            this._guide_tween1 = Laya.Tween.to(p1, { x: p2.x, y: p2.y, update: new Laya.Handler(this, this.updateGuide1Point, [p1]) }, 2000, Laya.Ease.quintInOut, Laya.Handler.create(this, () => {
                this.playGuide1Complete(p1);
            }));
            let p3 = a.createPoint(this._owner);
            let p4 = b.createPoint(this._owner);
            this._tips.x = p3.x;
            this._tips.y = p3.y;
            this._tips.visible = true;
            if (this._guide_tween2) {
                this._guide_tween2.clear();
            }
            this._guide_tween2 = Laya.Tween.to(this._tips, { x: p4.x, y: p4.y }, 2000, Laya.Ease.quintInOut);
        }
        updateGuide1Point(p1) {
            if (this._guide_point.distance(p1.x, p1.y) > 5) {
                this._guide_point.x = p1.x;
                this._guide_point.y = p1.y;
                this.checkTriggerArrow(p1.x, p1.y);
                this.updateArrows(p1.x, p1.y);
            }
        }
        playGuide1Complete(p1) {
            this.hideFocus();
            this.hideArrows();
            this.playGuide1();
        }
        playGuide2() {
            if (this._guide2_p1) {
                Laya.Tween.clearTween(this._guide2_p1);
            }
            this._guide_ing = true;
            let a = this.getMapBlock(8);
            let b = this.getMapBlock(4);
            let c = this.getMapBlock(5);
            let p1 = a.createPoint(Laya.stage);
            let p2 = b.createPoint(Laya.stage);
            let p3 = c.createPoint(Laya.stage);
            this._guide2_p1 = new Laya.Point(p1.x, p1.y);
            this._guide_point = new Laya.Point(p1.x, p1.y);
            if (this._guide_tween1) {
                this._guide_tween1.clear();
            }
            GameUtil.tweenPromise(this._guide2_p1, { x: p2.x, y: p2.y, update: new Laya.Handler(this, this.updateGuide2Point, [this._guide2_p1]) }, 2000, Laya.Ease.quintInOut).then(() => {
                this._guide_point = new Laya.Point(p1.x, p1.y);
                return GameUtil.tweenPromise(this._guide2_p1, { x: p3.x, y: p3.y, update: new Laya.Handler(this, this.updateGuide2Point, [this._guide2_p1]) }, 1000, Laya.Ease.quintInOut);
            }).then(() => {
                this.playGuide2Complete(this._guide2_p1);
            });
            let tips_p1 = a.createPoint(this._owner);
            let tips_p2 = b.createPoint(this._owner);
            let tips_p3 = c.createPoint(this._owner);
            this._tips.x = tips_p1.x;
            this._tips.y = tips_p1.y;
            this._tips.visible = true;
            Laya.Tween.clearTween(this._tips);
            GameUtil.tweenPromise(this._tips, { x: tips_p2.x, y: tips_p2.y }, 2000, Laya.Ease.quintInOut).then(() => {
                return GameUtil.tweenPromise(this._tips, { x: tips_p3.x, y: tips_p3.y }, 1000, Laya.Ease.quintInOut);
            });
        }
        updateGuide2Point(p1) {
            if (this._guide_point.distance(p1.x, p1.y) > 5) {
                this._guide_point.x = p1.x;
                this._guide_point.y = p1.y;
                this.checkTriggerArrow(p1.x, p1.y);
                this.updateArrows(p1.x, p1.y);
            }
        }
        playGuide2Complete(p1) {
            this.hideFocus();
            this.hideArrows();
            this.playGuide2();
        }
        getNeighboursByFaction(cur, faction) {
            let all = DataMgr.mapMgr.getBlockAll();
            let neighbours = new Map();
            this.findAllNeighbours(cur, all, new Map(), neighbours);
            let targets = [];
            neighbours.forEach((value, key) => {
                if (value.faction == faction) {
                    targets.push(value);
                }
            });
            return targets;
        }
        findAllNeighbours(cur, all, flags, neighbours) {
            for (let i = 0; i < cur.neighbour.length; i++) {
                if (cur.neighbour[i] > 0 && !flags.get(cur.neighbour[i])) {
                    let block = all.get(cur.neighbour[i]);
                    flags.set(cur.neighbour[i], true);
                    if (block == null) {
                        logI("不存在块:" + cur.neighbour[i]);
                        continue;
                    }
                    if (block.faction == cur.faction) {
                        this.findAllNeighbours(block, all, flags, neighbours);
                    }
                    else {
                        neighbours.set(block.id, block);
                    }
                }
            }
        }
        findPath(cur, blocks, tar_id, flags) {
            for (let i = 0; i < cur.neighbour.length; i++) {
                if (cur.neighbour[i] > 0 && !flags.get(cur.neighbour[i])) {
                    let block = blocks.get(cur.neighbour[i]);
                    flags.set(cur.neighbour[i], true);
                    if (block == null) {
                        logI("没找到:" + cur.neighbour[i]);
                        continue;
                    }
                    logI("寻找的块:" + block.name);
                    if (block.id == tar_id) {
                        logI("找到目标:" + block.name);
                        return true;
                    }
                    return this.findPath(block, blocks, tar_id, flags);
                }
            }
            return false;
        }
    }
    class MoveData {
        constructor(soldier, tween, tar_faction) {
            this.soldier = soldier;
            let data = soldier.getSoldierData();
            if (data) {
                this.cur_faction = data.faction;
            }
            this.tween = tween;
            this.tar_faction = tar_faction;
            this.trigger = false;
        }
        destroy() {
            this.soldier = null;
            this.tween.clear();
            this.tween = null;
        }
    }

    class ScaleButton {
        constructor(button) {
            this.scaleTime = 100;
            this._button = button;
            this.__scaleX = button.scaleX;
            this.__scaleY = button.scaleY;
            this._button.on(Laya.Event.MOUSE_DOWN, this, this.onClientDown);
            this._button.on(Laya.Event.MOUSE_UP, this, this.onClientUp);
            this._button.on(Laya.Event.MOUSE_OUT, this, this.onClientUp);
        }
        onClientDown() {
            Laya.Tween.to(this._button, { scaleX: this.__scaleX * 0.8, scaleY: this.__scaleY * 0.8 }, this.scaleTime);
        }
        onClientUp() {
            Laya.Tween.to(this._button, { scaleX: this.__scaleX, scaleY: this.__scaleX }, this.scaleTime);
        }
    }

    class MapBlock extends Laya.Script {
        constructor() {
            super();
            this.faction = null;
            this.count = 0;
            this.ai = BlockAI.easy;
            this._around_list = [];
            this._rate = 60;
            this._race_count = 0;
            this.r = 45;
            this.x = 150;
            this.y = 150;
            this.num = 0;
        }
        onEnable() {
            DataMgr.evetMgr.on(MiniGameEvent.SOLDIER_ATTACK_BLOCK, this, this.onBeAttacked);
            DataMgr.evetMgr.on(MiniGameEvent.SOLDIER_MERGE_BLOCK, this, this.onMerge);
        }
        onUpdate() {
            if (BlockFaction.neutral == this._block.faction) {
                return;
            }
            if (this._block.count >= GameConstant.BLOCK_COUNT_MAX) {
                return;
            }
            this._race_count++;
            if (this._race_count < GameConstant.GAME_COUNT_DELAY) {
                return;
            }
            if (this._race_count % this._rate == 0) {
                this.updateCount();
            }
        }
        init(block, soldier_prefab, map_sp) {
            this._block = block;
            this._soldier_prefab = soldier_prefab;
            this._owner = this.owner;
            this._init_pos = this.owner.getChildByName("init_pos");
            this._bg = this.owner.getChildByName("bg");
            this._text = this.owner.getChildByName("count");
            this._around_pos = new Laya.Sprite();
            this.updateFaction();
            this.refreshCount();
            this.createDefaultSoldier();
            this.initAround(map_sp);
        }
        initAround(map_sp) {
            let p1 = this.createPoint(map_sp);
            for (let i = 0; i < 20; i++) {
                this.num = this.num + 20;
                var a = Math.sin(this.num * Math.PI / 180) * this.r;
                var b = Math.cos(this.num * Math.PI / 180) * this.r;
                let new_x = (p1.x) + b;
                let new_y = (p1.y) + a;
                let p2 = new Laya.Point(new_x, new_y);
                this._around_list.push(p2);
            }
        }
        updateFaction() {
            this._bg.loadImage("mini_game/27/game/map_" + this._block.faction + ".png");
        }
        updateCount(value = -1) {
            let levelUp = this._block.updateCount(value);
            if (levelUp) {
                this._soldier_s.onSoldierChange();
                this.playEffectCircle();
            }
            this.refreshCount();
        }
        refreshCount() {
            this._text.text = this._block.count + "";
        }
        createPoint(parent) {
            return this._init_pos.localToGlobal(new Laya.Point(0, 0), true, parent);
        }
        createAroundPoint(from) {
            let distance = 0;
            let use_p;
            for (let i = 0; i < this._around_list.length; i++) {
                let p1 = this._around_list[i];
                let temp = p1.distance(from.x, from.y);
                if (distance == 0) {
                    distance = temp;
                    use_p = p1;
                }
                if (temp < distance) {
                    distance = temp;
                    use_p = p1;
                }
            }
            this._around_pos.x = use_p.x;
            this._around_pos.y = use_p.y;
            return use_p;
        }
        createDefaultSoldier() {
            this._soldier = Laya.Pool.getItemByCreateFun("soldier", this._soldier_prefab.create, this._soldier_prefab);
            this._soldier_s = this._soldier.getComponent(Soldier);
            let target_parent = this._owner.parent;
            let pos = this.createPoint(target_parent);
            this._soldier_s.init(this._block, target_parent, pos);
        }
        takeOutSoldier() {
            let takeOut = this._block.count;
            this.updateCount(0);
            return takeOut;
        }
        onMerge(soldier) {
            if (!this._block || this._block.id != soldier.target.id) {
                return;
            }
            if (soldier.isMerge()) {
                this.merge(soldier);
            }
        }
        onBeAttacked(soldier, tar) {
            if (!this._block || this._block.id != tar.id) {
                return;
            }
            if (soldier.isMerge()) {
                this.merge(soldier);
                return;
            }
            this.attack(soldier);
        }
        merge(soldier) {
            this.updateCount(this._block.count + soldier.count);
            this.playEffectCircle();
            logI("合并成功:" + soldier.name);
            DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_CLEAR_LOCK, [soldier]);
        }
        attack(soldier) {
            let obj = this._block.beAttacked(soldier.count);
            this.refreshCount();
            if (obj["lv_chnage"] == true) {
                this._soldier_s.onSoldierChange();
            }
            if (obj["success"] == false) {
                logI("占领失败");
                DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_CLEAR_LOCK, [soldier]);
                return;
            }
            logI("占领成功");
            this._block.changeData(soldier);
            this.updateFaction();
            this._soldier_s.onSoldierChange();
            this._soldier_s.playWuEffect();
            DataMgr.evetMgr.event(MiniGameEvent.SOLDIER_CLEAR_LOCK, [soldier]);
        }
        isFaction(faction) {
            return this._block.faction == faction;
        }
        getBlockId() {
            return this._block.id;
        }
        getBlock() {
            return this._block;
        }
        getBlockName() {
            return this._block.name;
        }
        playEffectCircle() {
            if (!this._circle_sp) {
                this._circle_sp = inner.MiniGameSpineMgr.createSpine(GameConstant.GAME_ID, GameConstant.getEffectRes(GameEffect.gquan));
            }
            this._circle_sp.stop();
            this._circle_sp.playByIndex(0, false);
            this._owner.addChild(this._circle_sp);
            this._circle_sp.pos(this._init_pos.x, this._init_pos.y - 20);
            SoundMgr.getInstance().playSound(GameConstant.SOUND_SHOOT);
        }
        destorySpine() {
            if (this._circle_sp) {
                this._circle_sp.destroy();
                this._circle_sp.removeSelf();
                this._circle_sp = null;
            }
        }
        onDisable() {
            DataMgr.evetMgr.off(MiniGameEvent.SOLDIER_MERGE_BLOCK, this, this.onMerge);
            DataMgr.evetMgr.off(MiniGameEvent.SOLDIER_ATTACK_BLOCK, this, this.onBeAttacked);
        }
        onDestroy() {
            this.destorySpine();
        }
    }

    class GameUI extends ui.mini_game.GameUI {
        constructor() {
            super();
            this._level = 0;
            this.loadedCount = 0;
            this.btnReturn.on(Laya.Event.CLICK, this, this.onReturn);
            this.btnReset.on(Laya.Event.CLICK, this, this.onReset);
            new ScaleButton(this.btnReturn);
            new ScaleButton(this.btnReset);
        }
        onEnable() {
        }
        onReset() {
            this.startGame(this._level);
        }
        onReturn() {
            this.closeUI();
            this.event(GameEvent.GAME_EXIT);
        }
        init(curCfg) {
            let self = this;
            SoundMgr.getInstance().musicStateUpdate(kr.MiniGameCtr.isMusicOn);
            GameUtil.targetVisble([self.funt_level], false);
            GameUtil.targetVisble([self.btnReset, self.btnReturn, self.btnShop], false);
            self.onDebug();
            self.destoryBlock();
            self.removeEvent();
            self.addEvent();
            this.loadedCount = 0;
            let prefab_uri = GameConstant.getMapRes(curCfg.map_id);
            Laya.loader.load(prefab_uri, Laya.Handler.create(self, self.onPrefabLoaded));
            let soldier_uri = GameConstant.getPrefab("soldier");
            Laya.loader.load(soldier_uri, Laya.Handler.create(self, self.onPrefabLoaded));
            self.btnReturn.visible = true;
            self.btnReset.visible = true;
            self.funt_level.visible = true;
            self.funt_level.value = `${self._level}`;
            if (curCfg.level <= 5) {
                self.btnReturn.visible = false;
            }
        }
        addEvent() {
            let self = this;
            self.on(Laya.Event.MOUSE_DOWN, self, self.onTouchDown);
            DataMgr.evetMgr.on(MiniGameEvent.BLOCK_GAME_RESULT, self, self.onGameResult);
        }
        removeEvent() {
            let self = this;
            self.off(Laya.Event.MOUSE_DOWN, self, self.onTouchDown);
            DataMgr.evetMgr.off(MiniGameEvent.BLOCK_GAME_RESULT, self, self.onGameResult);
        }
        onTouchDown(e) {
            this._map_script.stopGuide();
            this.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            this.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        onTouchMove(e) {
            if (DataMgr.mapMgr.gameIsRuning()) {
                this._map_script.checkTriggerArrow(Laya.stage.mouseX, Laya.stage.mouseY);
                this._map_script.updateArrows(Laya.stage.mouseX, Laya.stage.mouseY);
            }
        }
        onMouseUp(e) {
            if (DataMgr.mapMgr.gameIsRuning()) {
                this._map_script.checkArrowAttack(Laya.stage.mouseX, Laya.stage.mouseY);
            }
            this.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            this.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.off(Laya.Event.MOUSE_OUT, this, this.onMouseUp);
        }
        onPrefabLoaded() {
            if (this.loadedCount < 1) {
                this.loadedCount++;
                return;
            }
            this.initMap(this._curCfg);
        }
        initMap(curCfg) {
            let map_uri = GameConstant.getMapRes(curCfg.map_id);
            this._prefab_map = Laya.loader.getRes(map_uri);
            let soldier_uri = GameConstant.getPrefab("soldier");
            this._prefab_soldier = Laya.loader.getRes(soldier_uri);
            this._map_sp = Laya.Pool.getItemByCreateFun("mapPrefab", this._prefab_map.create, this._prefab_map);
            this._map_sp.y = 300;
            this.mapBox.addChild(this._map_sp);
            this._map_script = this._map_sp.getComponent(MapScript);
            this._map_script.init(curCfg, this._prefab_soldier);
            for (let i = 0; i < GameConstant.BLOCK_NAMES.length; i++) {
                let id = i + 1;
                let name = GameConstant.BLOCK_NAMES[i];
                let block = this._map_sp.getChildByName(name);
                let block_s = block.getComponent(MapBlock);
                if (block_s.faction == "" || block_s.faction == null) {
                    this._map_sp.removeChild(block);
                    continue;
                }
                this._map_script.addMapBlock(id, block_s);
                let data = new BlockData(id, name, block_s.faction, block_s.count, block_s.ai);
                DataMgr.mapMgr.addBlock(data);
                block_s.init(data, this._prefab_soldier, this._map_sp);
            }
            if (GameConstant.GUIDE_1 == curCfg.map_id) {
                this._map_script.playGuide1();
            }
            if (GameConstant.GUIDE_2 == curCfg.map_id) {
                this._map_script.playGuide2();
            }
            DataMgr.mapMgr.gameStart();
        }
        startGame(level) {
            this._level = level;
            this._curCfg = CfgMgr.getLevelCfg(level);
            DataMgr.mapMgr.initData(this._curCfg);
            this.init(this._curCfg);
        }
        onGameResult(is_victory) {
            this.onGmaeEnd(is_victory);
        }
        onGmaeEnd(is_victory) {
            if (is_victory) {
                this.closeUI();
                this.event(GameEvent.GAME_WIN);
            }
            else {
                this.closeUI();
                this.event(GameEvent.GAME_FAIL);
            }
        }
        closeUI() {
            Laya.timer.clearAll(this);
            this.removeEvent();
            this.destoryBlock();
        }
        destoryBlock() {
            if (this.mapBox) {
                this.mapBox.removeChildren(0, this.mapBox.numChildren - 1);
            }
            if (this._map_sp) {
                this._map_sp.destroy();
                this._map_sp = null;
                this._map_script = null;
            }
        }
        onDebug() {
        }
    }

    class Game_27 extends kr.BaseMiniGame {
        constructor() {
            super();
            this.gameId = 27;
            this._uiJsonPath = "";
        }
        initialization() {
            GameConstant.pathFix = "mini_game/" + GameConstant.GAME_ID + "/";
            this._uiJsonPath = GameConstant.pathFix + "ui.json";
            Laya.loader.load(GameConstant.pathFix + `cfg.json`, Laya.Handler.create(this, this.onConfigLoaded));
        }
        onConfigLoaded(cfg) {
            CfgMgr.parseCfgData(cfg);
            Laya.loader.load(this.getAtalsList(), Laya.Handler.create(this, this.onAtalsLoaded));
        }
        onAtalsLoaded() {
            Laya.loader.load(this._uiJsonPath, Laya.Handler.create(this, this.onUILoaded));
            DataMgr.init();
        }
        onUILoaded() {
            var uiJson = Laya.loader.getRes(this._uiJsonPath);
            this.setUiMap(uiJson);
            Laya.Scene.setUIMap(this._uiJsonPath);
            this.initGameUI();
            this.sendReady();
        }
        initGameUI() {
            this._gameUI = new GameUI();
            this.addChild(this._gameUI);
            this._gameUI.on(GameEvent.GAME_WIN, this, this.sendResult, [true]);
            this._gameUI.on(GameEvent.GAME_FAIL, this, this.sendResult, [false]);
            this._gameUI.on(GameEvent.GAME_EXIT, this, this.gameExit);
            logI("kr.MiniGameCtr.isMusicOn " + kr.MiniGameCtr.isMusicOn);
            SoundMgr.getInstance().musicStateUpdate(kr.MiniGameCtr.isMusicOn);
            kr.MiniGameCtr.on(kr.MiniGameEvent.MUSIC_STAT_UPDATE, this, this.onMusicUpdate);
        }
        onMusicUpdate() {
            logI(" ... onMusicUpdate ...");
            SoundMgr.getInstance().musicStateUpdate(kr.MiniGameCtr.isMusicOn);
        }
        sendResult(isWind) {
            SoundMgr.getInstance().stopBgMusic();
            var info = new kr.LevelResultInfo();
            info.isWin = isWind;
            info.gameId = this.gameId;
            info.level = this._startGameInfo.level;
            this.sendGameResult(info);
        }
        gameExit() {
            SoundMgr.getInstance().stopBgMusic();
            this.backToMain();
        }
        getAtalsList() {
            return [
                `${GameConstant.pathFix}game.atlas`
            ];
        }
        onShow() {
            if (this._gameUI) {
                if (this.parent) {
                    this.height = this.parent.height;
                    this._gameUI.height = this._gameUI.parent.height;
                }
            }
        }
        startGame(info) {
            this._startGameInfo = info;
            this._gameUI.startGame(info.level);
        }
        onHide() {
            SoundMgr.getInstance().stopAllSound();
        }
        _removeEvent() {
            if (this._gameUI) {
                this._gameUI.off(GameEvent.GAME_WIN, this, this.sendResult);
                this._gameUI.off(GameEvent.GAME_FAIL, this, this.sendResult);
                this._gameUI.off(GameEvent.GAME_EXIT, this, this.backToMain);
            }
        }
        onDestory() {
            this._removeEvent();
        }
        static init() {
            var reg = Laya.ClassUtils.regClass;
            reg("script/GameUI.ts", GameUI);
            reg("script/MapScript.ts", MapScript);
            reg("script/MapBlock.ts", MapBlock);
            reg("script/Soldier.ts", Soldier);
            reg("script/MapArrow.ts", MapArrow);
        }
    }
    window["Game_27"] = Game_27;
    Game_27.init();

    exports.Game_27 = Game_27;

    return exports;

}({}));
